import org.bouncycastle.crypto.digests.SHA256Digest;
import org.bouncycastle.crypto.generators.PKCS5S2ParametersGenerator;
import org.bouncycastle.crypto.params.KeyParameter;

import javax.crypto.Cipher;
import javax.crypto.SecretKey;
import javax.crypto.spec.IvParameterSpec;
import javax.crypto.spec.SecretKeySpec;
import java.nio.charset.Charset;
import java.util.StringTokenizer;

public class CryptoUtil {

    private static final Charset characterSetUtf8 = Charset.forName("UTF-8");

    private static final Charset characterSetAscii = Charset.forName("US-ASCII");

    public static Cipher cipherFactory(String secretKey, String secretIv, String salt, int mode) {
        try {


            //PBKDF2WithHmacSHA256
            final PKCS5S2ParametersGenerator gen = new PKCS5S2ParametersGenerator(new SHA256Digest());
            gen.init(secretKey.getBytes(characterSetAscii), salt.getBytes(characterSetAscii), 2);
            final byte[] dk = ((KeyParameter) gen.generateDerivedParameters(256)).getKey();
            SecretKey secret = new SecretKeySpec(dk, "AES");



            Cipher cipher = Cipher.getInstance("AES/CBC/PKCS5Padding");
            cipher.init(mode, secret, new IvParameterSpec(secretIv.getBytes(characterSetAscii)));
            return cipher;
        } catch (Exception e) {
            System.out.printf("CryptoUtil.cipherFactory - fail to create cipher %s %s\n", e.getMessage(), mode);
            return null;
        }
    }



    public static byte[] encode(Cipher cipher, byte[] payload) {
        if (payload == null || cipher == null) {
            return null;
        }
        try {
            return cipher.doFinal(payload);
        } catch (Exception e) {
            System.out.printf("CryptoUtil.decode - fail to encode %s %s", payload, e.getMessage());
        }
        return null;
    }

    public static byte[] decode(Cipher cipher, byte[] encrypted) {
        if (encrypted== null) {
            return null;
        }
        try {
            return cipher.doFinal(encrypted);
        } catch (Exception e) {
            System.out.printf("CryptoUtil.decode - fail to decode {} {}", encrypted, e.getMessage());
        }
        return null;
    }

    public static String encode64(Cipher cipher, String payload) {
        if (payload == null || cipher == null) {
            return null;
        }
        try {
            byte[] encrypted = cipher.doFinal(payload.getBytes(characterSetUtf8));
            return String.valueOf(Base64.encode(encrypted));
        } catch (Exception e) {
            System.out.printf("CryptoUtil.decode - fail to encode %s %s", payload, e.getMessage());
        }
        return null;
    }

    public static String decode64(Cipher cipher, String encryptedSessionInfo) {
        if (encryptedSessionInfo == null) {
            return null;
        }
        try {
            byte[] byteStr = Base64.decode(encryptedSessionInfo);
            return new String(cipher.doFinal(byteStr), characterSetUtf8);

        } catch (Exception e) {
            System.out.printf("CryptoUtil.decode - fail to decode {} {}", encryptedSessionInfo, e.getMessage());
        }
        return null;
    }

    public static String encode58(Cipher cipher, String payload) {
        if (payload == null || cipher == null) {
            return null;
        }
        try {
            byte[] encrypted = cipher.doFinal(payload.getBytes(characterSetUtf8));
            return Base58.encodeChecked(encrypted);
        } catch (Exception e) {
            System.out.printf("CryptoUtil.decode - fail to encode %s %s", payload, e.getMessage());
        }
        return null;
    }

    public static String decode58(Cipher cipher, String encryptedSessionInfo) {
        if (encryptedSessionInfo == null) {
            return null;
        }
        try {
            byte[] byteStr = Base58.decodeChecked(encryptedSessionInfo);
            if(byteStr==null){
                return null;
            }
            return new String(cipher.doFinal(byteStr), characterSetUtf8);

        } catch (Exception e) {
            System.out.printf("CryptoUtil.decode - fail to decode {} {}", encryptedSessionInfo, e.getMessage());
        }
        return null;
    }

    /**
     * timestamp
     *
     * @param encodedSessionInfo
     * @return
     */
    public static String getAccessTokenIfValid(Cipher cipher, String encodedSessionInfo) {
        if (encodedSessionInfo == null || cipher == null) {
            return null;
        }

        String decodedSessionInfo = decode58(cipher, encodedSessionInfo);
        StringTokenizer splitted = decodedSessionInfo == null ? null : new StringTokenizer(decodedSessionInfo, ":");

        if (splitted == null || splitted.countTokens() != 2) {
            return null;
        }
        return decodedSessionInfo;
    }
}